package org.apache.hadoop.dfs;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.WritableUtils;

/**
 * NetFlow is used to describe an on-going shuffle flow.
 * 
 * It is not synchronized because the usage is diverse in TaskTracker &
 * ReduceTask.
 * 
 * @author sko
 *
 */
public class NetFlow implements Writable, Cloneable {

  private String src;
  private String dst;
  // the following two can uniquely identify a flow
  private String mapTaskId; // this is a Map task attempt id
  private String reduceTaskId; // this is a Reduce task attempt id

  private long total; // total bytes
  private long bytes; // bytes transmitted
  
  public NetFlow() {}
  
  public NetFlow(String src, String mapTaskId, 
      String dst, String reduceTaskId, long totalLen, long len) {
    this.reset(src, mapTaskId, dst, reduceTaskId, totalLen, len);
  }
  
  public void reset(String src, 
      String mapTaskId, String dst, String reduceTaskId,
      long totalLen, long len) {
    this.src = src;
    this.mapTaskId = mapTaskId;
    this.dst = dst;
    this.reduceTaskId = reduceTaskId;
    this.total = totalLen;
    this.bytes = len;
  }
  
  public String getMapTaskId() {
    return mapTaskId;
  }
  
  public String getReduceTaskId() {
    return reduceTaskId;
  }
  
  public void setDst(String dst) {
    this.dst = dst;
  }
  
  public void addBytes(long len) {
    bytes += len;
  }
  
  public void setBytes(long len) {
    bytes = len;
  }
      
  public long getBytes() {
    return bytes;
  }
  
  public void setTotalBytes(long len) {
    total = len;
  }
  
  public long getTotalBytes() {
    return total;
  }
  
  /** 
   * Serialize the fields of this object to <code>out</code>.
   * 
   * @param out <code>DataOuput</code> to serialize this object into.
   * @throws IOException
   */
  public void write(DataOutput out) throws IOException {
    if (src == null) {
      out.writeBoolean(false);
    } else {
      out.writeBoolean(true);
      Text.writeString(out, src);
    }
    if (mapTaskId == null) {
      out.writeBoolean(false);
    } else {
      out.writeBoolean(true);
      Text.writeString(out, mapTaskId);
    }
    if (dst == null) {
      out.writeBoolean(false);
    } else {
      out.writeBoolean(true);
      Text.writeString(out, dst);
    }
    if (reduceTaskId == null) {
      out.writeBoolean(false);
    } else {
      out.writeBoolean(true);
      Text.writeString(out, reduceTaskId);
    }
    WritableUtils.writeVLong(out, total);
    WritableUtils.writeVLong(out, bytes);
  }

  /** 
   * Deserialize the fields of this object from <code>in</code>.  
   * 
   * <p>For efficiency, implementations should attempt to re-use storage in the 
   * existing object where possible.</p>
   * 
   * @param in <code>DataInput</code> to deseriablize this object from.
   * @throws IOException
   */
  public void readFields(DataInput in) throws IOException {
    if (in.readBoolean()) {
      src = Text.readString(in);
    } else {
      src = null;
    }
    if (in.readBoolean()) {
      mapTaskId = Text.readString(in);
    } else {
      mapTaskId = null;
    }
    if (in.readBoolean()) {
      dst = Text.readString(in);
    } else {
      dst = null;
    }
    if (in.readBoolean()) {
      reduceTaskId = Text.readString(in);
    } else {
      reduceTaskId = null;
    }
    total = WritableUtils.readVLong(in);
    bytes = WritableUtils.readVLong(in);
  }
  
  public String toString() {
    return String.valueOf(this.bytes) + "/" +
      String.valueOf(this.total) + 
      " from " + src + " (" +
      mapTaskId + ")" + " to " + dst + " (" + reduceTaskId + ")";
  }
  
  public NetFlow clone() {
    return new NetFlow(src, mapTaskId, dst, reduceTaskId, total, bytes);
  }
}